/* 
 * Copyright (C) 2001-2007 Jacek Sieka, arnetheduck on gmail point com
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include "stdinc.h"
#include "DCPlusPlus.h"

#include "LogManager.h"

LogManager::LogManager() {
	logOptions[UPLOAD][FILE]		= SettingsManager::LOG_FILE_UPLOAD;
	logOptions[UPLOAD][FORMAT]		= SettingsManager::LOG_FORMAT_POST_UPLOAD;
    logOptions[DOWNLOAD][FILE]		= SettingsManager::LOG_FILE_DOWNLOAD;
	logOptions[DOWNLOAD][FORMAT]		= SettingsManager::LOG_FORMAT_POST_DOWNLOAD;
	logOptions[CHAT][FILE]			= SettingsManager::LOG_FILE_MAIN_CHAT;
	logOptions[CHAT][FORMAT]		= SettingsManager::LOG_FORMAT_MAIN_CHAT;
	logOptions[PM][FILE]			= SettingsManager::LOG_FILE_PRIVATE_CHAT;
	logOptions[PM][FORMAT]			= SettingsManager::LOG_FORMAT_PRIVATE_CHAT;
	logOptions[SYSTEM][FILE]		= SettingsManager::LOG_FILE_SYSTEM;
	logOptions[SYSTEM][FORMAT]		= SettingsManager::LOG_FORMAT_SYSTEM;
	logOptions[STATUS][FILE]		= SettingsManager::LOG_FILE_STATUS;
	logOptions[STATUS][FORMAT]		= SettingsManager::LOG_FORMAT_STATUS;
};

void LogManager::log(LogArea area, StringMap& params) throw() {
	string path = SETTING(LOG_DIRECTORY);
	string msg;
	
	path += Util::formatParams(getSetting(area, FILE), params, true);
	msg = Util::formatParams(getSetting(area, FORMAT), params, false);

	log(path, msg);
}

void LogManager::message(const string& msg, bool logit/*= true*/) {
	if(BOOLSETTING(LOG_SYSTEM) && logit) {
		StringMap params;
		params["message"] = msg;
		log(LogManager::SYSTEM, params);
	}
	
	time_t t = GET_TIME();
	{
		Lock l(cs);
		// Keep the last 100 messages (completely arbitrary number...)
		while(lastLogs.size() > 100)
			lastLogs.pop_front();
		lastLogs.push_back(make_pair(t, msg));
	}
	fire(LogManagerListener::Message(), t, msg);
}

void LogManager::log(const string& area, const string& msg) throw() {
	Lock l(cs);
	try {
		string aArea = Util::validateFileName(area);
		File::ensureDirectory(aArea);
		File f(aArea, File::WRITE, File::OPEN | File::CREATE);
		f.setEndPos(0);
		if(f.getPos() == 0) {
			f.write("\xef\xbb\xbf");
		}
		f.write(msg + "\r\n");
	} catch (const FileException&) {
		// ...
	}
}

/* Some parts Copyright (C) 2002-2007 Petr Kozelka, PPK at czdc dot org */