/* 
 * Copyright (C) 2002-2003 Trpaslik
 * Copyright (C) 2003-2004 createagaga
 * Copyright (C) 2002-2007 Petr Kozelka, PPK at czdc dot org
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include "stdafx.h"
#include "Resource.h"
#include "../client/DCPlusPlus.h"
#include "ChatCtrl.h"
#include "../client/FavoriteManager.h"
#include "AGEmoticonSetup.h"
#include "../client/DebugManager.h"

extern CAGEmoticonSetup* g_pEmoticonsSetup;

static const TCHAR* Links[] = { _T("http://"), _T("https://"), _T("www."), _T("ftp://"), 
	_T("magnet:?"), _T("dchub://"), _T("irc://"), _T("ed2k://"), _T("mms://") };
static const TCHAR* LinkBreaks = _T(" \n\t");
tstring ChatCtrl::sSelectedUser = Util::emptyStringT;
tstring ChatCtrl::sSelectedLine = Util::emptyStringT;
tstring ChatCtrl::sSelectedIP = Util::emptyStringT;

ChatCtrl::ChatCtrl() {
	m_boAutoScroll = true;
	client = NULL;
	long lMask = GetEventMask();
	SetEventMask(lMask | ENM_LINK);
}

ChatCtrl::~ChatCtrl() {
	client = NULL;
}

void ChatCtrl::SetClient(Client * pClient) {
	client = pClient;
}

void ChatCtrl::AdjustTextSize() {
	// We want to limit the buffer to 25000 characters...after that, w95 becomes sad...
	if(GetWindowTextLength() > 25000) {
		SetRedraw(FALSE);
		SetSel(0, LineIndex(LineFromChar(2000)));
		ReplaceSel(Util::emptyStringT.c_str());
		SetRedraw(TRUE);
	}
}

void ChatCtrl::AppendText(const Identity& i, const tstring& sMyNick, const tstring& sTime, const tstring& sIP, const tstring& sMsg, CHARFORMAT2& cf, bool bUseEmo/* = true*/) {
	SetRedraw(FALSE);
	long lSelBeginSaved, lSelEndSaved;
	GetSel(lSelBeginSaved, lSelEndSaved);
	POINT cr;
	SendMessage(EM_GETSCROLLPOS, 0, (LPARAM)&cr);

	long lSelBegin = 0, lSelEnd = 0;

	PARAFORMAT2 pf;
	memset(&pf, 0, sizeof(PARAFORMAT2));
	pf.dwMask = PFM_STARTINDENT; 
	pf.dxStartIndent = 0;

	// Insert TimeStamp and format with default style
	if(sTime.empty() == false) {
		lSelEnd = lSelBegin = GetTextLengthEx(GTL_NUMCHARS);
		SetSel(lSelEnd, lSelEnd);
		ReplaceSel(sTime.c_str(), false);
		lSelEnd = GetTextLengthEx(GTL_NUMCHARS);
		SetSel(lSelBegin, lSelEnd - 1);
		SetSelectionCharFormat(WinUtil::m_TextStyleTimestamp);
		SetParaFormat(pf);
	}
	
	// Insert IP
	if(sIP.empty() == false) {
		lSelEnd = lSelBegin = GetTextLengthEx(GTL_NUMCHARS);
		SetSel(lSelEnd, lSelEnd);
		ReplaceSel((sIP+_T(' ')).c_str(), false);
		lSelEnd = GetTextLengthEx(GTL_NUMCHARS);
		SetSel(lSelBegin, lSelEnd - 1);
		SetSelectionCharFormat(cf);
	}

	TCHAR *sText = (TCHAR *)sMsg.c_str();
	size_t iTextLen = sMsg.size();
	tstring sAuthor = Text::toT(i.getNick());
	bool bMyMess = i.getUser() == ClientManager::getInstance()->getMe();
	if(sAuthor.empty() == false) {
		unsigned int iLen = 0, iAuthorLen = sAuthor.size()+1;
		if(sMsg[0] == _T('*'))
            iLen = 1;
            
		lSelEnd = lSelBegin = GetTextLengthEx(GTL_NUMCHARS);
		SetSel(lSelEnd, lSelEnd);
		TCHAR temp = sText[iAuthorLen+iLen];
		sText[iAuthorLen+iLen] = _T('\0');
		ReplaceSel(sText, false);
		sText[iAuthorLen+iLen] = temp;
		sText += iAuthorLen+iLen;
		iTextLen -= iAuthorLen+iLen;
		if(bMyMess == true) {
			SetSel(lSelBegin, lSelBegin+iLen+1);
			SetSelectionCharFormat(WinUtil::m_ChatTextMyOwn);
			SetSel(lSelBegin+iLen+1, lSelBegin+iLen+iAuthorLen);
			SetSelectionCharFormat(WinUtil::m_TextStyleMyNick);
		} else {
			bool isFavorite = FavoriteManager::getInstance()->isFavoriteUser(i.getUser());

			if(BOOLSETTING(BOLD_AUTHOR_MESS) || i.isOp() || isFavorite) {
				SetSel(lSelBegin, lSelBegin+iLen+1);
				SetSelectionCharFormat(cf);
				SetSel(lSelBegin+iLen+1, lSelBegin+iLen+iAuthorLen);
				if(isFavorite){
					SetSelectionCharFormat(WinUtil::m_TextStyleFavUsers);
				} else if(i.isOp()) {
					SetSelectionCharFormat(WinUtil::m_TextStyleOPs);
				} else {
					SetSelectionCharFormat(WinUtil::m_TextStyleBold);
				}
			} else {
				SetSel(lSelBegin, lSelBegin+iLen+iAuthorLen);
				SetSelectionCharFormat(cf);
            }
		}
	} else {
        switch(sMsg[0]) {
            case _T('<'): {
				TCHAR *temp;
                if((temp = _tcschr((TCHAR *)sMsg.c_str()+1, _T('>'))) != NULL) {
                    int iAuthorLen = temp - sMsg.c_str();
		            lSelEnd = lSelBegin = GetTextLengthEx(GTL_NUMCHARS);
		            SetSel(lSelEnd, lSelEnd);
					TCHAR temp = sText[iAuthorLen];
		            sText[iAuthorLen] = _T('\0');
            		ReplaceSel(sText, false);
					sText[iAuthorLen] = temp;
					sText += iAuthorLen;
					iTextLen -= iAuthorLen;
        			if(BOOLSETTING(BOLD_AUTHOR_MESS)) {
        				SetSel(lSelBegin, lSelBegin+1);
        				SetSelectionCharFormat(cf);
                        SetSel(lSelBegin+1, lSelBegin+iAuthorLen);
        				SetSelectionCharFormat(WinUtil::m_TextStyleBold);
        			} else {
        				SetSel(lSelBegin, lSelBegin+iAuthorLen);
        				SetSelectionCharFormat(cf);
                    }
				}
                break;
            }
            case _T('*'): {
				TCHAR *temp;
                if(sMsg[1] == _T(' ') && (temp = _tcschr((TCHAR *)sMsg.c_str()+2, _T(' '))) != NULL) {
                    int iAuthorLen = (temp - sMsg.c_str())-1;
		            lSelEnd = lSelBegin = GetTextLengthEx(GTL_NUMCHARS);
		            SetSel(lSelEnd, lSelEnd);
					TCHAR temp = sText[iAuthorLen+1];
		            sText[iAuthorLen+1] = _T('\0');
            		ReplaceSel(sText, false);
					sText[iAuthorLen+1] = temp;
					sText += iAuthorLen+1;
					iTextLen -= iAuthorLen+1;
        			if(BOOLSETTING(BOLD_AUTHOR_MESS)) {
        				SetSel(lSelBegin, lSelBegin+2);
        				SetSelectionCharFormat(cf);
      					SetSel(lSelBegin+2, lSelBegin+1+iAuthorLen);
        				SetSelectionCharFormat(WinUtil::m_TextStyleBold);
        			} else {
        				SetSel(lSelBegin, lSelBegin+1+iAuthorLen);
        				SetSelectionCharFormat(cf);
                    }
				}
                break;
            }
            default:
                break;
        }
	}

	// Cachry machry, aby tam byly vzdy stejne oddelovace radku, 
	// nejlip se osvedcil pouze "\n", jinak ujizdi obarveni nicku
	size_t p = 0;
	AutoArray<TCHAR> sMsgLowerArr(iTextLen+1);
	for(size_t t = 0; t < iTextLen; t++) {
		switch(sText[t]) {
			case _T('\r'):
				sText[p] = _T('\n');
				sMsgLowerArr[p] = _T('\n');
				if(sText[t+1] == _T('\n')) {
					t++;
				}
				break;
			case _T('\0'):
				sText[p] = _T('0');
				sMsgLowerArr[p] = _T('0');
				break;
			default:
				if(p != t) {
					sText[p] = sText[t];
				}
				sMsgLowerArr[p] = _totlower(sText[t]);
				break;
		}
		p++;
	}

	if(iTextLen != p) {
		iTextLen = p;
		sText[p] = _T('\0');
	}
	sMsgLowerArr[p] = _T('\0');
	TCHAR *sMsgLower = sMsgLowerArr;
	
	// cachry machry a maj s toho vylizt smajlove
	if(g_pEmoticonsSetup->getUseEmoticons() && BOOLSETTING(USE_EMOTICONS) && bUseEmo) {
		CAGEmoticon::List& Emoticons = g_pEmoticonsSetup->EmoticonsList;
		int smiles = 0;
		bool bUrlOnEnd = false;
		while(true) {
			TCHAR *rpl = NULL;
			CAGEmoticon::Ptr pFoundedEmoticon = NULL;
			int len = iTextLen;
			for(CAGEmoticon::Iter pEmoticon = Emoticons.begin(); pEmoticon != Emoticons.end(); ++pEmoticon) {
				int nIdxFound = -1;
				TCHAR *txt = Util::strstr(sText, (*pEmoticon)->GetEmoticonText().c_str(), nIdxFound);
				if((txt < rpl && txt) || rpl == NULL && txt) {
					if(len > nIdxFound) {
						rpl = txt;
						pFoundedEmoticon = (*pEmoticon);
						len = nIdxFound;
					}
				}
			}

			if(rpl && (smiles < SETTING(MAX_EMOTICONS))) {
                TCHAR temp = rpl[pFoundedEmoticon->GetEmoticonText().size()];
                rpl[pFoundedEmoticon->GetEmoticonText().size()] = _T('\0');
                sMsgLower[(rpl-sText)+pFoundedEmoticon->GetEmoticonText().size()] = _T('\0');
				long lUrlEnd = AppendTextOnly(sMyNick, sText, sMsgLower, (rpl+pFoundedEmoticon->GetEmoticonText().size())-sText, cf, bMyMess, bUrlOnEnd);
				rpl[pFoundedEmoticon->GetEmoticonText().size()] = temp;
				sMsgLower[(rpl-sText)+pFoundedEmoticon->GetEmoticonText().size()] = _totlower(temp);
				lUrlEnd == 0 ? bUrlOnEnd = true : bUrlOnEnd = false;
            	if(lUrlEnd == -1 || (size_t)lUrlEnd > pFoundedEmoticon->GetEmoticonText().size()) {
    				COLORREF clrBkColor = WinUtil::m_ChatTextGeneral.crBackColor;
    				if(bMyMess == true)
                        clrBkColor = WinUtil::m_ChatTextMyOwn.crBackColor;
                        
    				HBITMAP hbmNext = pFoundedEmoticon->GetEmoticonBmp(clrBkColor);
					lSelEnd = GetTextLengthEx(GTL_NUMCHARS);
    				SetSel(lSelEnd-pFoundedEmoticon->GetEmoticonText().size(), lSelEnd);
    				CImageDataObject::InsertBitmap(GetOleInterface(), hbmNext);
    				smiles++;
                }
				iTextLen -= (rpl-sText) + pFoundedEmoticon->GetEmoticonText().size();
				sMsgLower += (rpl-sText) + pFoundedEmoticon->GetEmoticonText().size();
				sText += (rpl-sText) + pFoundedEmoticon->GetEmoticonText().size();
			} else {
				if(iTextLen > 0) {
					AppendTextOnly(sMyNick, sText, sMsgLower, iTextLen, cf, bMyMess, bUrlOnEnd);
				}
				break;
			}
		}
	} else {
		AppendTextOnly(sMyNick, sText, sMsgLower, iTextLen, cf, bMyMess);
	}
	
	lSelEnd = GetTextLengthEx(GTL_NUMCHARS);
	if(lSelEnd != 0) {
		SetSel(lSelEnd, lSelEnd);
		ReplaceSel(_T("\n"), false);
	}
	
	SetSel(lSelBeginSaved, lSelEndSaved);
	SetScrollPos(&cr);
	EndRedrawAppendTextOnly();
}

void ChatCtrl::AppendText(const tstring& sMyNick, const tstring& sTime, const tstring& sMsg, CHARFORMAT2& cf, const tstring& sAuthor, int iAuthorLen, bool isMe, bool bUseEmo/* = true*/) {
	SetRedraw(FALSE);
	long lSelBeginSaved, lSelEndSaved;
	GetSel(lSelBeginSaved, lSelEndSaved);
	POINT cr;
	SendMessage(EM_GETSCROLLPOS, 0, (LPARAM)&cr);

	long lSelBegin = 0, lSelEnd = 0;

	PARAFORMAT2 pf;
	memset(&pf, 0, sizeof(PARAFORMAT2));
	pf.dwMask = PFM_STARTINDENT; 
	pf.dxStartIndent = 0;

	// Insert TimeStamp and format with default style
	if(sTime.empty() == false) {
		lSelEnd = lSelBegin = GetTextLengthEx(GTL_PRECISE);
		SetSel(lSelEnd, lSelEnd);
		ReplaceSel(sTime.c_str(), false);
		lSelEnd = GetTextLengthEx(GTL_PRECISE);
		SetSel(lSelBegin, lSelEnd - 1);
		SetSelectionCharFormat(WinUtil::m_TextStyleTimestamp);
		SetParaFormat(pf);
	}
    	
	bool myMess = false;
	TCHAR *sText = (TCHAR *)sMsg.c_str();
	size_t iTextLen = sMsg.size();
	if(iAuthorLen > 0) {
		int iLen = 0;
		if(isMe)
			iLen = 1;

		lSelEnd = lSelBegin = GetTextLengthEx(GTL_PRECISE);
		SetSel(lSelEnd, lSelEnd);
		TCHAR temp = sText[iAuthorLen+iLen];
		sText[iAuthorLen+iLen] = _T('\0');
		ReplaceSel(sText, false);
		sText[iAuthorLen+iLen] = temp;
		sText += iAuthorLen+iLen;
		iTextLen -= iAuthorLen+iLen;
		if(_tcsicmp(sMyNick.c_str(), sAuthor.c_str()) == 0) {
			myMess = true;
			SetSel(lSelBegin, lSelBegin+iLen+1);
			SetSelectionCharFormat(WinUtil::m_ChatTextMyOwn);
			SetSel(lSelBegin+iLen+1, lSelBegin+iLen+iAuthorLen);
			SetSelectionCharFormat(WinUtil::m_TextStyleMyNick);
		} else {
			bool isFavorite = false;
			FavoriteManager::FavoriteMap ul = FavoriteManager::getInstance()->getFavoriteUsers();
			for(FavoriteManager::FavoriteMap::iterator i = ul.begin(); i != ul.end(); ++i) {
				FavoriteUser aUser = i->second;
				if(_tcsicmp(Text::toT(aUser.getNick()).c_str(), sAuthor.c_str()) == 0) {
					isFavorite = true;
					break;
				}
			}
			bool isOP = false;
			if(isFavorite == false) {
				if(client != NULL)  {
					OnlineUser* ou = client->findUser(Text::fromT(sAuthor));				
    				if(ou != NULL) {
    					isOP = ou->getIdentity().isOp();
    				}
                }
			}
			if(BOOLSETTING(BOLD_AUTHOR_MESS) || isOP || isFavorite) {
				SetSel(lSelBegin, lSelBegin+iLen+1);
				SetSelectionCharFormat(cf);
				if(isFavorite){
					SetSel(lSelBegin+iLen+1, lSelBegin+iLen+iAuthorLen);
					SetSelectionCharFormat(WinUtil::m_TextStyleFavUsers);
				} else if(isOP) {
					SetSel(lSelBegin+iLen+1, lSelBegin+iLen+iAuthorLen);
					SetSelectionCharFormat(WinUtil::m_TextStyleOPs);
				} else {
					SetSel(lSelBegin+iLen+1, lSelBegin+iLen+iAuthorLen);
					SetSelectionCharFormat(WinUtil::m_TextStyleBold);
				}
			} else {
				SetSel(lSelBegin, lSelBegin+iLen+iAuthorLen);
				SetSelectionCharFormat(cf);
            }
		}
	}

	// Cachry machry, aby tam byly vzdy stejne oddelovace radku, 
	// nejlip se osvedcil pouze "\n", jinak ujizdi obarveni nicku
	size_t p = 0;
	AutoArray<TCHAR> sMsgLowerArr(iTextLen+1);
	for(size_t t = 0; t < iTextLen; t++) {
		switch(sText[t]) {
			case _T('\r'):
				sText[p] = _T('\n');
				sMsgLowerArr[p] = _T('\n');
				if(sText[t+1] == _T('\n')) {
					t++;
				}
				break;
			case _T('\0'):
				sText[p] = _T('0');
				sMsgLowerArr[p] = _T('0');
				break;
			default:
				if(p != t) {
					sText[p] = sText[t];
				}
				sMsgLowerArr[p] = _totlower(sText[t]);
				break;
		}
		p++;
	}

	if(iTextLen != p) {
		iTextLen = p;
		sText[p] = _T('\0');
	}
	sMsgLowerArr[p] = _T('\0');
	TCHAR *sMsgLower = sMsgLowerArr;

	// cachry machry a maj s toho vylizt smajlove
	if(g_pEmoticonsSetup->getUseEmoticons() && BOOLSETTING(USE_EMOTICONS) && bUseEmo) {
		CAGEmoticon::List& Emoticons = g_pEmoticonsSetup->EmoticonsList;
		int smiles = 0;;
		bool bUrlOnEnd = false;
		while(true) {
			TCHAR *rpl = NULL;
			CAGEmoticon::Ptr pFoundedEmoticon = NULL;
			int len = iTextLen;
			for(CAGEmoticon::Iter pEmoticon = Emoticons.begin(); pEmoticon != Emoticons.end(); ++pEmoticon) {
				int nIdxFound = -1;
				TCHAR *txt = Util::strstr(sText, (*pEmoticon)->GetEmoticonText().c_str(), nIdxFound);
				if((txt < rpl && txt) || rpl == NULL && txt) {
					if(len > nIdxFound) {
						rpl = txt;
						pFoundedEmoticon = (*pEmoticon);
						len = nIdxFound;
					}
				}
			}

			if(rpl && (smiles < SETTING(MAX_EMOTICONS))) {
                TCHAR temp = rpl[pFoundedEmoticon->GetEmoticonText().size()];
                rpl[pFoundedEmoticon->GetEmoticonText().size()] = _T('\0');
				sMsgLower[(rpl-sText)+pFoundedEmoticon->GetEmoticonText().size()] = _T('\0');
				long lUrlEnd = AppendTextOnly(sMyNick, sText, sMsgLower, (rpl+pFoundedEmoticon->GetEmoticonText().size())-sText, cf, myMess, bUrlOnEnd);
				rpl[pFoundedEmoticon->GetEmoticonText().size()] = temp;
				sMsgLower[(rpl-sText)+pFoundedEmoticon->GetEmoticonText().size()] = _totlower(temp);
				lUrlEnd == 0 ? bUrlOnEnd = true : bUrlOnEnd = false;
				if(lUrlEnd == -1 || (size_t)lUrlEnd > pFoundedEmoticon->GetEmoticonText().size()) {
    				COLORREF clrBkColor = WinUtil::m_ChatTextGeneral.crBackColor;
    				if(myMess)
                        clrBkColor = WinUtil::m_ChatTextMyOwn.crBackColor;
                        
    				HBITMAP hbmNext = pFoundedEmoticon->GetEmoticonBmp(clrBkColor);
    				lSelEnd = GetTextLengthEx(GTL_PRECISE);
    				SetSel(lSelEnd-pFoundedEmoticon->GetEmoticonText().size(), lSelEnd);
    				CImageDataObject::InsertBitmap(GetOleInterface(), hbmNext);
    				smiles++;
                }
				iTextLen -= (rpl-sText) + pFoundedEmoticon->GetEmoticonText().size();
				sMsgLower += (rpl-sText) + pFoundedEmoticon->GetEmoticonText().size();
				sText += (rpl-sText) + pFoundedEmoticon->GetEmoticonText().size();
			} else {
				if(iTextLen > 0) {
					AppendTextOnly(sMyNick, sText, sMsgLower, iTextLen, cf, myMess, bUrlOnEnd);
				}
				break;
			}
		}
	} else {
		AppendTextOnly(sMyNick, sText, sMsgLower, iTextLen, cf, myMess);
	}
	
	lSelEnd = GetTextLengthEx(GTL_NUMCHARS);
	if(lSelEnd != 0) {
		SetSel(lSelEnd, lSelEnd);
		ReplaceSel(_T("\n"), false);
	}
	
	SetSel(lSelBeginSaved, lSelEndSaved);
	SetScrollPos(&cr);
	EndRedrawAppendTextOnly();
}

long ChatCtrl::AppendTextOnly(const tstring& sMyNick, const TCHAR *sText, TCHAR *sMsgLower, const size_t iTextLen, CHARFORMAT2& cf, bool bMyMess, bool bUrlOnEnd/* = false*/) {
	long lSelBegin = 0, lSelEnd = 0, lUrlEnd = -1;

	// Insert text at the end
	lSelEnd = lSelBegin = GetTextLengthEx(GTL_NUMCHARS);
	SetSel(lSelEnd, lSelEnd);
	ReplaceSel(sText, false);

    lSelEnd = GetTextLengthEx(GTL_NUMCHARS);
	SetSel(lSelBegin, lSelEnd);
    if(bMyMess == true) {
		// Moje vlastni zprava
		SetSelectionCharFormat(WinUtil::m_ChatTextMyOwn);
	} else {
		SetSelectionCharFormat(cf);
	}
	
	// Set char format for rest of url
	if(bUrlOnEnd == true) {
		TCHAR *linkbreak = _tcspbrk((TCHAR *)sText, LinkBreaks);
        long linkEnd = (linkbreak != NULL ? linkbreak-sText : iTextLen);
		SetSel(lSelBegin, lSelBegin + linkEnd);
		SetSelectionCharFormat(WinUtil::m_TextStyleURL);
		if(lUrlEnd == -1 || lUrlEnd > (lSelEnd-(lSelBegin + linkEnd))) {
			lUrlEnd = lSelEnd-(lSelBegin + linkEnd);
		}
	}

	// Zvyrazneni vsech URL a nastaveni "klikatelnosti"
	long lSearchFrom = 0;
	for(size_t i = 0; i < (sizeof(Links) / sizeof(Links[0])); i++) {
		TCHAR *temp = _tcsstr(sMsgLower+lSearchFrom, Links[i]);
		while(temp != NULL) {
			TCHAR *linkbreak = _tcspbrk(temp, LinkBreaks);
            long linkEnd = (linkbreak != NULL ? linkbreak-sMsgLower : iTextLen);
			SetSel(lSelBegin + (temp-sMsgLower), lSelBegin + linkEnd);
			SetSelectionCharFormat(WinUtil::m_TextStyleURL);
			temp = _tcsstr(sMsgLower+linkEnd, Links[i]);
			if(lUrlEnd == -1 || lUrlEnd > (lSelEnd-(lSelBegin + linkEnd))) {
				lUrlEnd = lSelEnd-(lSelBegin + linkEnd);
			}
		}
	}

	// Zvyrazneni vsech vyskytu vlastniho nicku
	lSearchFrom = 0; 
    long lMyNickStart = -1, lMyNickEnd = -1;

	if(sMyNick.size() != 0) {
		tstring sNick = Text::toLower(sMyNick);
		while(true) {
			TCHAR *temp = _tcsstr(sMsgLower+lSearchFrom, sNick.c_str());
			if(temp == NULL) 
				break;
	
			lMyNickStart = (temp-sMsgLower);
			lMyNickEnd = lMyNickStart + sNick.size();
			SetSel(lSelBegin + lMyNickStart, lSelBegin + lMyNickEnd);
			SetSelectionCharFormat(WinUtil::m_TextStyleMyNick);
			lSearchFrom = lMyNickEnd;
		}
	}

	// Zvyrazneni vsech vyskytu nicku Favorite useru
	FavoriteManager::FavoriteMap ul = FavoriteManager::getInstance()->getFavoriteUsers();
	for(FavoriteManager::FavoriteMap::iterator i = ul.begin(); i != ul.end(); ++i) {
		FavoriteUser pUser = i->second;
		lSearchFrom = 0;
		tstring sNick = Text::toT(Text::toLower(pUser.getNick())); 

		while(true) {
			TCHAR *temp = _tcsstr(sMsgLower+lSearchFrom, sNick.c_str());
			if(temp == NULL) 
				break;

			lMyNickStart = (temp-sMsgLower);
			lMyNickEnd = lMyNickStart + sNick.size();
			SetSel(lSelBegin + lMyNickStart, lSelBegin + lMyNickEnd);
			SetSelectionCharFormat(WinUtil::m_TextStyleFavUsers);
			lSearchFrom = lMyNickEnd;
		}
	}
	return lUrlEnd;
}

void ChatCtrl::AppendStatusText(const tstring& sTime, tstring sMsg, CHARFORMAT2& cf) {
	SetRedraw(FALSE);
	long lSelBeginSaved, lSelEndSaved;
	GetSel(lSelBeginSaved, lSelEndSaved);
	POINT cr;
	SendMessage(EM_GETSCROLLPOS, 0, (LPARAM)&cr);

	long lSelBegin = 0, lSelEnd = 0;

	PARAFORMAT2 pf;
	memset(&pf, 0, sizeof(PARAFORMAT2));
	pf.dwMask = PFM_STARTINDENT; 
	pf.dxStartIndent = 0;

	// Insert TimeStamp and format with default style
	if(sTime.empty() == false) {
		lSelEnd = lSelBegin = GetTextLengthEx(GTL_PRECISE);
		SetSel(lSelEnd, lSelEnd);
		ReplaceSel(sTime.c_str(), false);
		lSelEnd = GetTextLengthEx(GTL_PRECISE);
		SetSel(lSelBegin, lSelEnd - 1);
		SetSelectionCharFormat(WinUtil::m_TextStyleTimestamp);
		SetParaFormat(pf);
	}
    	
	// Insert text at the end
	lSelEnd = lSelBegin = GetTextLengthEx(GTL_NUMCHARS);
	SetSel(lSelEnd, lSelEnd);
	ReplaceSel(sMsg.c_str(), false);
	lSelEnd = GetTextLengthEx(GTL_PRECISE);
	SetSel(lSelBegin, lSelEnd);
	SetSelectionCharFormat(cf);

	lSelEnd = GetTextLengthEx(GTL_NUMCHARS);
	if(lSelEnd != 0) {
		SetSel(lSelEnd, lSelEnd);
		ReplaceSel(_T("\n"), false);
	}
	
	SetSel(lSelBeginSaved, lSelEndSaved);
	SetScrollPos(&cr);
	EndRedrawAppendTextOnly();
}


void ChatCtrl::EndRedrawAppendTextOnly() {
	if(m_boAutoScroll) {
		SetSel(0, 0);
		ScrollCaret();
		SetSel(-1, -1);
		ScrollCaret();
	}
	// Force window to redraw
	SetRedraw(TRUE);
	InvalidateRect(NULL);
}

bool ChatCtrl::HitNick(POINT p, tstring &sNick, int &iBegin, int &iEnd) {
	if(client == NULL) 
		return false;

	int iCharPos = CharFromPos(p), line = LineFromChar(iCharPos), len = LineLength(iCharPos) + 1;
	long lSelBegin = 0, lSelEnd = 0;
	if(len < 3)
		return false;

	// Metoda FindWordBreak nestaci, protoze v nicku mohou byt znaky povazovane za konec slova
	int iFindBegin = LineIndex(line), iEnd1 = LineIndex(line) + LineLength(iCharPos);

	for(lSelBegin = iCharPos; lSelBegin >= iFindBegin; lSelBegin--) {
		if(FindWordBreak(WB_ISDELIMITER, lSelBegin))
			break;
	}
	lSelBegin++;
	for(lSelEnd = iCharPos; lSelEnd < iEnd1; lSelEnd++) {
		if(FindWordBreak(WB_ISDELIMITER, lSelEnd))
			break;
	}

	len = lSelEnd - lSelBegin;
	if(len <= 0)
		return false;

	AutoArray<TCHAR> tsTemp(len+1);
	GetTextRange(lSelBegin, lSelEnd, tsTemp);
	tstring sText(tsTemp, len);

	int iLeft = 0, iRight = 0, iCRLF = sText.size(), iPos = sText.find(_T('<'));
	if(iPos >= 0) {
		iLeft = iPos + 1;
		iPos = sText.find(_T('>'), iLeft);
		if(iPos < 0) 
			return false;
			
		iRight = iPos - 1;
		iCRLF = iRight - iLeft + 1;
	} else {
		iLeft = 0;
	}

	tstring sN = sText.substr(iLeft, iCRLF);
	if(sN.size() == 0)
		return false;

	if(client->findUser(Text::fromT(sN)) != NULL) {
		sNick = sN;
		iBegin = lSelBegin + iLeft;
		iEnd = lSelBegin + iLeft + iCRLF;
		return true;
	}
    
	// Jeste pokus odmazat eventualni koncovou ':' nebo '>' 
	// Nebo pro obecnost posledni znak 
	// A taky prvni znak 
	// A pak prvni i posledni :-)
	if(iCRLF > 1) {
		sN = sText.substr(iLeft, iCRLF - 1);
		if(client->findUser(Text::fromT(sN)) != NULL) {
			sNick = sN;
       		iBegin = lSelBegin + iLeft;
       		iEnd = lSelBegin + iLeft + iCRLF - 1;
			return true;
		}

		sN = sText.substr(iLeft + 1, iCRLF - 1);
		if(client->findUser(Text::fromT(sN)) != NULL) {
       		sNick = sN;
			iBegin = lSelBegin + iLeft + 1;
			iEnd = lSelBegin + iLeft + iCRLF;
			return true;
		}

		sN = sText.substr(iLeft + 1, iCRLF - 2);
		if(client->findUser(Text::fromT(sN)) != NULL) {
			sNick = sN;
       		iBegin = lSelBegin + iLeft + 1;
   			iEnd = lSelBegin + iLeft + iCRLF - 1;
			return true;
		}
	}	
	return false;
}

bool ChatCtrl::HitIP(POINT p, tstring &sIP, int &iBegin, int &iEnd) {
	int iCharPos = CharFromPos(p), len = LineLength(iCharPos) + 1;
	if(len < 3)
		return false;

	DWORD lPosBegin = FindWordBreak(WB_LEFT, iCharPos);
	DWORD lPosEnd = FindWordBreak(WB_RIGHTBREAK, iCharPos);
	len = lPosEnd - lPosBegin;

	AutoArray<TCHAR> tsTemp(len+1);
	GetTextRange(lPosBegin, lPosEnd, tsTemp);
	for(int i = 0; i < len; i++) {
		if(((tsTemp[i] == 0) || (tsTemp[i] == _T('.')) || ((tsTemp[i] >= _T('0')) && (tsTemp[i] <= _T('9')))) == false) {
			return false;
		}
	}
	tstring sText(tsTemp, len);

	sText += _T('.');
	int iFindBegin = 0, iPos = -1, iEnd2 = 0;
	bool boOK = true;
	for(int i = 0; i < 4; i++) {
		iPos = sText.find(_T('.'), iFindBegin);
		if(iPos < 0) {
			boOK = false;
			break;
		}
		iEnd2 = atoi(Text::fromT(sText.substr(iFindBegin)).c_str());
		if((iEnd2 < 0) || (iEnd2 > 255)) {
			boOK = false;
			break;
		}
		iFindBegin = iPos + 1;
	}

	if(boOK) {
		sIP = sText.substr(0, iPos);
		iBegin = lPosBegin;
		iEnd = lPosEnd;
	}
	return boOK;
}

bool ChatCtrl::HitURL() {
	long lSelBegin = 0, lSelEnd = 0;
	GetSel(lSelBegin, lSelEnd);
	bool boOK = false;

	CHARFORMAT2 cfSel;
	cfSel.cbSize = sizeof(cfSel);
    
	GetSelectionCharFormat(cfSel);
	if(cfSel.dwEffects & CFE_LINK) {
		boOK = true;
	}
	return boOK;
}

tstring ChatCtrl::LineFromPos(POINT p) {
	int iCharPos = CharFromPos(p), line = LineFromChar(iCharPos), len = LineLength(iCharPos) + 1;
	if(len < 3) {
		return Util::emptyStringT;
	}
	AutoArray<TCHAR> tsTemp(len+1);
	GetLine(line, tsTemp, len);
	tstring x(tsTemp, len-1);
	return x;
}

void ChatCtrl::GoToEnd() {
	if(m_boAutoScroll) {
		SetRedraw(false);
		SetSel(0, 0);
		ScrollCaret();
		SetSel(-1, -1);
		ScrollCaret();
		SetRedraw(true);
		InvalidateRect(NULL);
	}
}

bool ChatCtrl::GetAutoScroll() {
	return m_boAutoScroll;
}

void ChatCtrl::SetAutoScroll(bool boAutoScroll) {
	m_boAutoScroll = boAutoScroll;
	if(boAutoScroll)
		GoToEnd();
}

LRESULT ChatCtrl::OnRButtonDown(POINT pt) {
	long lSelBegin = 0, lSelEnd = 0; tstring sSel;

	sSelectedLine = LineFromPos(pt);
	sSelectedUser = Util::emptyStringT;
	sSelectedIP = Util::emptyStringT;
	
	// Po kliku dovnitr oznaceneho textu si zkusime poznamenat pripadnej nick ci ip...
	// jinak by nam to neuznalo napriklad druhej klik na uz oznaceny nick =)
	GetSel(lSelBegin, lSelEnd);
	int iCharPos = CharFromPos(pt), iBegin = 0, iEnd = 0;
	if((lSelEnd > lSelBegin) && (iCharPos >= lSelBegin) && (iCharPos <= lSelEnd)) {
		if(HitIP(pt, sSel, iBegin, iEnd)) {
			sSelectedIP = sSel;
		} else if(HitNick(pt, sSel, iBegin, iEnd)) {
			sSelectedUser = sSel;
		}
		return 1;
	}
	
	if(HitIP(pt, sSel, iBegin, iEnd)) {
        // Po kliku do IP oznacit IP
		sSelectedIP = sSel;
		SetSel(iBegin, iEnd);
		InvalidateRect(NULL);
	} else if(HitNick(pt, sSel, iBegin, iEnd)) {
        // Po kliku na Nick oznacit Nick
		sSelectedUser = sSel;
		SetSel(iBegin, iEnd);
		InvalidateRect(NULL);
	}
	return 1;
}

void ChatCtrl::ClearChat() {
	SetWindowText(Util::emptyStringT.c_str());
}