//
// Author : Hani Atassi  (atassi@arabteam2000.com)
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "ImageDataObject.h"

// Static member functions

void CImageDataObject::InsertBitmap(IRichEditOle* pRichEditOle, HBITMAP hBitmap) {
    LPLOCKBYTES lpLockBytes = NULL;
	SCODE sc = ::CreateILockBytesOnHGlobal(NULL, TRUE, &lpLockBytes);
	
	if(sc != S_OK) {
        if(lpLockBytes != NULL)
            lpLockBytes->Release();

        DeleteObject(hBitmap);
        pRichEditOle->Release();
        return;
    }

	// Initialize a Storage Object
	IStorage *pStorage = NULL;	
	sc = ::StgCreateDocfileOnILockBytes(lpLockBytes,
		STGM_SHARE_EXCLUSIVE|STGM_CREATE|STGM_READWRITE, 0, &pStorage);
	
	if(sc != S_OK) {
        if(pStorage != NULL)
            pStorage->Release();

        if(lpLockBytes != NULL)
            lpLockBytes->Release();

		DeleteObject(hBitmap);
        pRichEditOle->Release();
        return;
	}
  	
	CImageDataObject pods;
	pods.SetBitmap(hBitmap);

	// Get the RichEdit container site
	IOleClientSite *pOleClientSite;	
	sc = pRichEditOle->GetClientSite(&pOleClientSite);

    if(SUCCEEDED(sc)) {
        IOleObject *pOleObject = pods.GetOleObject(pOleClientSite, pStorage);
        if(pOleObject != NULL) {
            OleSetContainedObject(pOleObject, TRUE);

            // Now Add the object to the RichEdit 
			REOBJECT reobject = {0};
            reobject.cbStruct = sizeof(REOBJECT);
	
            CLSID clsid;

            sc = pOleObject->GetUserClassID(&clsid);

    		if(sc != S_OK) {
    			dcdebug("Thrown OLE Exception: %d\n", sc);
    			return;
    		}

    		reobject.clsid = clsid;
    		reobject.cp = REO_CP_SELECTION;
    		reobject.dvaspect = DVASPECT_CONTENT;
    		reobject.dwFlags = REO_BELOWBASELINE;
    		reobject.poleobj = pOleObject;
    		reobject.polesite = pOleClientSite;
    		reobject.pstg = pStorage;

            // Insert the bitmap at the current location in the richedit control
            pRichEditOle->InsertObject(&reobject);

            // Release all unnecessary interfaces
            pOleObject->Release();
    	}

    	pOleClientSite->Release();
    }

	pStorage->Release();
	lpLockBytes->Release();
	DeleteObject(hBitmap);
	pRichEditOle->Release();
}

void CImageDataObject::SetBitmap(HBITMAP hBitmap) {
	STGMEDIUM stgm;
	stgm.tymed = TYMED_GDI;
	stgm.hBitmap = hBitmap;
	stgm.pUnkForRelease = NULL;

	FORMATETC fm;
	fm.cfFormat = CF_BITMAP;
	fm.ptd = NULL;
	fm.dwAspect = DVASPECT_CONTENT;
	fm.lindex = -1;
	fm.tymed = TYMED_GDI;

	this->SetData(&fm, &stgm, TRUE);		
}

IOleObject *CImageDataObject::GetOleObject(IOleClientSite *pOleClientSite, IStorage *pStorage) {
	IOleObject *pOleObject;

	if(::OleCreateStaticFromData(this, IID_IOleObject, OLERENDER_FORMAT, 
			&m_fromat, pOleClientSite, pStorage, (void **)&pOleObject) != S_OK) {
		return NULL;
	}

	return pOleObject;
}

/* Some parts Copyright (C) 2003-2004 createagaga */
/* Some parts Copyright (C) 2004-2007 Petr Kozelka, PPK at czdc dot org */