/* 
 * Copyright (C) 2004-2007 Petr Kozelka, PPK at czdc dot org
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include "stdafx.h"
#include "../client/DCPlusPlus.h"
#include "Resource.h"

#include "ThemesPage.h"
#include "../client/SettingsManager.h"
#include "WinUtil.h"
#include "MainFrm.h"

PropPage::TextItem ThemesPage::texts[] = {
	{ IDC_ENABLE_EMOTICONS, ResourceManager::SETCZDC_USE_CMAJLIES },
	{ IDC_SETCZDC_MAX_EMOTICONS, ResourceManager::SETCZDC_MAX_EMOTICONS },
	{ IDC_CZDC_EMOTICONFILE, ResourceManager::SETCZDC_EMOTICONSFILE },
	{ IDC_EMOTICONBROWSE, ResourceManager::BROWSE },
	{ IDC_EMOTICONREFRESH, ResourceManager::RELOAD },
	{ IDC_CZDC_EMOTICONS, ResourceManager::EMOTICONS },
	{ IDC_SETTINGS_SOUNDS, ResourceManager::SETTINGS_SOUNDS },
	{ IDC_PRIVATE_MESSAGE_BEEP, ResourceManager::SETTINGS_PM_BEEP },
	{ IDC_PRIVATE_MESSAGE_BEEP_OPEN, ResourceManager::SETTINGS_PM_BEEP_OPEN },
	{ IDC_CHANGE_COLOR, ResourceManager::SETTINGS_CHANGE_COLOR }, 
	{ IDC_USERLIST_COLORING, ResourceManager::USERLIST_COLORING },
	{ IDC_ENABLE_USERLIST_COLORS, ResourceManager::ENABLE_USERLIST_COLORS },
	{ 0, ResourceManager::SETTINGS_AUTO_AWAY }
};

PropPage::Item ThemesPage::items[] = {
	{ IDC_ENABLE_EMOTICONS, SettingsManager::USE_EMOTICONS, PropPage::T_BOOL },
	{ IDC_MAX_EMOTICONS, SettingsManager::MAX_EMOTICONS, PropPage::T_INT },
	{ IDC_PRIVATE_MESSAGE_BEEP, SettingsManager::PRIVATE_MESSAGE_BEEP, PropPage::T_BOOL },
	{ IDC_PRIVATE_MESSAGE_BEEP_OPEN, SettingsManager::PRIVATE_MESSAGE_BEEP_OPEN, PropPage::T_BOOL },
	{ IDC_ENABLE_USERLIST_COLORS, SettingsManager::USERLIST_COLORING, PropPage::T_BOOL },
	{ 0, 0, PropPage::T_END }
};

LRESULT ThemesPage::onInitDialog(UINT /*uMsg*/, WPARAM /*wParam*/, LPARAM /*lParam*/, BOOL& /*bHandled*/) {
	PropPage::translate((HWND)(*this), texts);
	PropPage::read((HWND)*this, items);

	CUpDownCtrl spin;
	spin.Attach(GetDlgItem(IDC_MAX_EMOTICONSSPIN));
	spin.SetRange32(1, 999);
	spin.Detach();

	normalColour = SETTING(NORMAL_COLOR);
	favoriteColour = SETTING(FAVORITE_COLOR);
	reservedSlotColour = SETTING(EXTRA_SLOT_COLOR);
	ignoredColour = SETTING(IGNORED_COLOR);
	fastColour = SETTING(FIREBALL_COLOR);
	serverColour = SETTING(SERVER_COLOR);
	activeColour = SETTING(ACTIVE_COLOR);
	pasiveColour = SETTING(PASIVE_COLOR);
	opColour = SETTING(OP_COLOR);

	n_lsbList.Attach(GetDlgItem(IDC_USERLIST_COLORS));
	n_Preview.Attach(GetDlgItem(IDC_PREVIEW));
	n_Preview.SetBackgroundColor(WinUtil::bgColor);
	n_lsbList.AddString(CTSTRING(COLOR_NORMAL));
	n_lsbList.AddString(CTSTRING(COLOR_FAVORITE));	
	n_lsbList.AddString(CTSTRING(COLOR_RESERVED));
	n_lsbList.AddString(CTSTRING(COLOR_IGNORED));
	n_lsbList.AddString(CTSTRING(COLOR_FAST));
	n_lsbList.AddString(CTSTRING(COLOR_SERVER));
	n_lsbList.AddString(CTSTRING(COLOR_OP));
	n_lsbList.AddString(CTSTRING(COLOR_ACTIVE));
	n_lsbList.AddString(CTSTRING(COLOR_PASIVE));
	n_lsbList.SetCurSel(0);

	refreshPreview(); 
	
	// Do specialized reading here
	SetDlgItemText(IDC_EMOTICONFILE, Text::toT(SETTING(EMOTICONFILE)).c_str());
	return TRUE;
}

void ThemesPage::write() {
	PropPage::write((HWND)*this, items);
	SettingsManager::getInstance()->set(SettingsManager::NORMAL_COLOR, normalColour);
	SettingsManager::getInstance()->set(SettingsManager::FAVORITE_COLOR, favoriteColour);
	SettingsManager::getInstance()->set(SettingsManager::EXTRA_SLOT_COLOR, reservedSlotColour);
	SettingsManager::getInstance()->set(SettingsManager::IGNORED_COLOR, ignoredColour);
	SettingsManager::getInstance()->set(SettingsManager::FIREBALL_COLOR, fastColour);
	SettingsManager::getInstance()->set(SettingsManager::SERVER_COLOR, serverColour);
	SettingsManager::getInstance()->set(SettingsManager::ACTIVE_COLOR, activeColour);
	SettingsManager::getInstance()->set(SettingsManager::PASIVE_COLOR, pasiveColour);
	SettingsManager::getInstance()->set(SettingsManager::OP_COLOR, opColour);
	
	// Do specialized writing here
	if(SETTING(MAX_EMOTICONS) < 1) settings->set(SettingsManager::MAX_EMOTICONS, 1);
	TCHAR buf[256];
	GetDlgItemText(IDC_EMOTICONFILE, buf, 256);
	string file = Text::fromT(buf);
	if(_stricmp(file.c_str(), SETTING(EMOTICONFILE).c_str()) != 0) {
		settings->set(SettingsManager::EMOTICONFILE, file);
		MainFrame::initEmoticons();
	}
}

LRESULT ThemesPage::onBrowse(WORD /*wNotifyCode*/, WORD /*wID*/, HWND /*hWndCtl*/, BOOL& /*bHandled*/) {
	TCHAR buf[MAX_PATH];

	GetDlgItemText(IDC_EMOTICONFILE, buf, MAX_PATH);
	tstring x = buf;

	if(WinUtil::browseFile(x, m_hWnd, false) == IDOK) {
		if(_tcscmp(buf, x.c_str()) != 0)
			::EnableWindow(GetDlgItem(IDC_EMOTICONREFRESH), FALSE);
		SetDlgItemText(IDC_EMOTICONFILE, x.c_str());
	}
	return 0;
}

LRESULT ThemesPage::onRefresh(WORD /*wNotifyCode*/, WORD /*wID*/, HWND /*hWndCtl*/, BOOL& /*bHandled*/) {
	MainFrame::initEmoticons();
	return 0;
}

LRESULT ThemesPage::onChangeColour(WORD /*wNotifyCode*/, WORD /*wID*/, HWND hWndCtl, BOOL& /*bHandled*/) {
	int index = n_lsbList.GetCurSel();
	if(index == -1) return 0;
	int colour = 0;
	switch(index) {
		case 0: colour = normalColour; break;
		case 1: colour = favoriteColour; break;
		case 2: colour = reservedSlotColour; break;
		case 3: colour = ignoredColour; break;
		case 4: colour = fastColour; break;
		case 5: colour = serverColour; break;
		case 7: colour = activeColour; break;
		case 8: colour = pasiveColour; break;
		case 6: colour = opColour; break;
		default: break;
	}
	CColorDialog d(colour, 0, hWndCtl);
	if(d.DoModal() == IDOK) {
		switch(index) {
			case 0: normalColour = d.GetColor(); break;
			case 1: favoriteColour = d.GetColor(); break;
			case 2: reservedSlotColour = d.GetColor(); break;
			case 3: ignoredColour = d.GetColor(); break;
			case 4: fastColour = d.GetColor(); break;
			case 5: serverColour = d.GetColor(); break;
			case 7: activeColour = d.GetColor(); break;
			case 8: pasiveColour = d.GetColor(); break;
			case 6: opColour = d.GetColor(); break;
			default: break;
		}
		refreshPreview();
	}
	return TRUE;
}

void ThemesPage::refreshPreview() {
	CHARFORMAT2 cf;
	n_Preview.SetWindowText(_T(""));
	cf.dwMask = CFM_COLOR;
	cf.dwEffects = 0;
	
	cf.crTextColor = normalColour;
	n_Preview.SetSelectionCharFormat(cf);
	n_Preview.AppendText(TSTRING(COLOR_NORMAL).c_str());

	cf.crTextColor = favoriteColour;
	n_Preview.SetSelectionCharFormat(cf);
	n_Preview.AppendText((_T("\r\n") + TSTRING(COLOR_FAVORITE)).c_str());
	
	cf.crTextColor = reservedSlotColour;
	n_Preview.SetSelectionCharFormat(cf);
	n_Preview.AppendText((_T("\r\n") + TSTRING(COLOR_RESERVED)).c_str());
	
	cf.crTextColor = ignoredColour;
	n_Preview.SetSelectionCharFormat(cf);
	n_Preview.AppendText((_T("\r\n") + TSTRING(COLOR_IGNORED)).c_str());
	
	cf.crTextColor = fastColour;
	n_Preview.SetSelectionCharFormat(cf);
	n_Preview.AppendText((_T("\r\n") + TSTRING(COLOR_FAST)).c_str());

	cf.crTextColor = serverColour;
	n_Preview.SetSelectionCharFormat(cf);
	n_Preview.AppendText((_T("\r\n") + TSTRING(COLOR_SERVER)).c_str());

	cf.crTextColor = opColour;
	n_Preview.SetSelectionCharFormat(cf);
	n_Preview.AppendText((_T("\r\n") + TSTRING(COLOR_OP)).c_str());
	
	cf.crTextColor = activeColour;
	n_Preview.SetSelectionCharFormat(cf);
	n_Preview.AppendText((_T("\r\n") + TSTRING(COLOR_ACTIVE)).c_str());

	cf.crTextColor = pasiveColour;
	n_Preview.SetSelectionCharFormat(cf);
	n_Preview.AppendText((_T("\r\n") + TSTRING(COLOR_PASIVE)).c_str());

	n_Preview.InvalidateRect(NULL);
}
